package com.inet.application.googleanalytics;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.nio.charset.StandardCharsets;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.util.HashMap;

import com.inet.application.googleanalytics.structure.GoogleAnalyticsConfigKeys;
import com.inet.authentication.LoginProcessor;
import com.inet.config.Configuration;
import com.inet.config.ConfigurationManager;
import com.inet.file.FileCombiner.RuntimeData;
import com.inet.lib.json.Json;
import com.inet.persistence.Persistence;

/**
 * AdHoc Configuration of the GA Plugin
 */
public class GoogleAnalyticsRuntimeData implements RuntimeData {

    /**
     * {@inheritDoc}
     */
    @Override
    public InputStream getDataStream() {

        StringBuilder string = new StringBuilder();

        Configuration current = ConfigurationManager.getInstance().getCurrent();
        if ( !current.get( GoogleAnalyticsConfigKeys.TRACKING_ID.getKey(), GoogleAnalyticsConfigKeys.TRACKING_ID.getDefault() ).isEmpty() ) {

            string.append( "GOGINFO = " );
            HashMap<String, Object> data = new HashMap<>();
            data.put( "trackingId", current.get( GoogleAnalyticsConfigKeys.TRACKING_ID.getKey(), GoogleAnalyticsConfigKeys.TRACKING_ID.getDefault() ) );
            data.put( "anonymizeIp", current.get( GoogleAnalyticsConfigKeys.ANONYMIZE_IP.getKey(), GoogleAnalyticsConfigKeys.ANONYMIZE_IP.getDefault()  ) );
            data.put( "pageview",  "" );

            HashMap<String, Object> options = new HashMap<>();
            data.put( "options", options );

            if ( Boolean.valueOf( current.get( GoogleAnalyticsConfigKeys.TRACK_USERS.getKey(), GoogleAnalyticsConfigKeys.TRACK_USERS.getDefault() ) ).booleanValue() && LoginProcessor.getCurrent() != null) {
                try {
                    byte[] digest = MessageDigest.getInstance( "MD5" ).digest( (Persistence.getRecoveryEnabledInstance().getUID() + " " + LoginProcessor.getCurrent().getLoginID()).getBytes( StandardCharsets.UTF_8) );

                    StringBuilder sb = new StringBuilder();
                    for(int i=0;i<digest.length;i++){
                        sb.append(Integer.toHexString(0xff & digest[i]));
                    }
                    options.put( "userId", sb.toString() );
                } catch( NoSuchAlgorithmException e ) {
                    // Do not track
                }
            }

            if ( !current.get( GoogleAnalyticsConfigKeys.COOKIE_DOMAIN.getKey(), GoogleAnalyticsConfigKeys.COOKIE_DOMAIN.getDefault() ).isEmpty() ) {
                data.put( "cookieDomain", current.get( GoogleAnalyticsConfigKeys.COOKIE_DOMAIN.getKey(), GoogleAnalyticsConfigKeys.COOKIE_DOMAIN.getDefault() ) );
                data.put( "legacyCookieDomain", current.get( GoogleAnalyticsConfigKeys.COOKIE_DOMAIN.getKey(), GoogleAnalyticsConfigKeys.COOKIE_DOMAIN.getDefault() ) );
            }

            string.append( new Json().toJson( data ) );
            string.append( ";" );
        }

        return new ByteArrayInputStream( string.toString().getBytes(StandardCharsets.UTF_8) );
    }

}
